#include "moduleHeader.h"
#include "mp3dec.h"
#include <stddef.h>

void RegisterAppMalloc(void * fp);
void RegisterAppFree(void * fp);
void *(*appRegisteredMalloc)(size_t n);
void (*appRegisteredFree)(void * fp);

void LibStartupInit(void * data);

/************************************************
 * This is table of exported library procedures.
 * Note how each entry is a combination of a 
 * string key and a function pointer. Also note
 * how the library exports a registering function
 * to use the applicatin malloc routine.
 ************************************************/

const T_PROC_DCPT exportFunctionsTbl[] = 
{
	{"HelixMP3InitDecoder",		(void *)MP3InitDecoder      },
	{"HelixMP3FreeDecoder",		(void *)MP3FreeDecoder      },
    {"HelixMP3Decode",          (void *)MP3Decode           },
    {"HelixMP3GetLastFrameInfo",(void *)MP3GetLastFrameInfo },
    {"HelixMP3GetNextFrameInfo",(void *)MP3GetNextFrameInfo },
    {"HelixMP3FindSyncWord",    (void *)MP3FindSyncWord     },
	{"HelixMP3RegMalloc",       (void *)RegisterAppMalloc},
	{"HelixMP3RegFree",         (void *)RegisterAppFree}
};	

/***************************************************************
 * This is the library module header. The MODULE_HEADER_ADDRESS,
 * MODULE_NAME and NUM_OF_EXPORTED_FUNCTIONS macros are 
 * defined in moduleHeader.h.
 ***************************************************************/

const T_MODULE_DYN_HDR __attribute__((address(HELIX_LIB_MODULE_HEADER_ADDRESS))) _ModuleLoadHdr =
{
	
	MODULE_NAME,			                        // name of the Module	
	LibStartupInit,			                        // Start up code
	NUM_OF_EXPORTED_FUNCTIONS(exportFunctionsTbl),	// This a macro which calculates how many
                                                    // functions the library exported.
	exportFunctionsTbl		                    // Export Procedure Table	
};	


/*************************************************************
 * This function calls the library startup initialization
 * routine defined in lib_Startup.S. The library startup
 * initialization routine initializes the bss and data sections
 * of the library. It then calls a user definable LibInit()
 * function.
 *************************************************************/ 
void LibStartupInit(void * data)
{
    /* This function is defined in lib_Startup.S */
    _initLibVariables();

    /* This function should be defined by the library
     * implementation.
     */

    LibInit(data); 
}

/****************************************
 * This function registers malloc supplied
 * by application
 ****************************************/
void RegisterAppMalloc(void * functionPtr)
{
    appRegisteredMalloc = (void *(*)(size_t))functionPtr;
}

/******************************************
 * This is the library malloc. Note that
 * we are not using the malloc from stdlib.
 * This is an example of how the library
 * uses the application heap and how the 
 * library uses the application registered
 * function.
 ******************************************/

void * malloc(size_t n)
{
    return(appRegisteredMalloc( n));
}

/****************************************
 * This function registers free supplied
 * by application
 ****************************************/
void RegisterAppFree(void * functionPtr)
{
    appRegisteredFree = (void (*)(void *))functionPtr;
}

/******************************************
 * This is the library free. Note that
 * we are not using the free from stdlib.
 * This is an example of how the library
 * uses the application heap and how the 
 * library uses the application registered
 * function.
 ******************************************/

void free(void * p)
{
    appRegisteredFree(p);
}

/*******************************************
* We have not used the LibInit function for
* any custom initialization. So this is left
* blank.
********************************************/
void LibInit(void * data)
{
}
